import bottle
from configparser import ConfigParser
import json
import os, time

from logger import Logger
from singleton import Singleton

class DummyProcessing(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')

        conf = ConfigParser()
        self.__updateCases()

        servicestub.route('/payment', method='POST', callback=self.payment)
        servicestub.route('/cancel', method='POST', callback=self.cancel)
        servicestub.route('/balance', method='POST', callback=self.balance)

    def payment(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('/payment')

    def cancel(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('/cancel')

    def balance(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('/balance')

    def createResponse(self, methodName):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            return None

        response = json.dumps(self.cases[methodName][0].get('response'))
        time.sleep(self.cases[methodName][0].get('timeout', 0))

        if 'count' in self.cases[methodName][0]:
            self.cases[methodName][0]['count'] -= 1
            if self.cases[methodName][0]['count'] == 0:
                del self.cases[methodName][0]

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

instance = DummyProcessing()
