import json
import os
import time
import xml.etree.ElementTree as ET
from typing import Dict, Any

import bottle
from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Manzana2019(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        self.path = os.path.dirname(os.path.abspath(__file__))
        self.preffix = ".//{http://loyalty.manzanagroup.ru/loyalty.xsd}"

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса Manzana2019')
        self.__updateCases()

        servicestub.route('/posprocessing.asmx', method='POST', callback=self.handler)

    def handler(self):
        self.__updateCases()
        root = ET.fromstring(bottle.request.body.read())
        action = "undefined"

        requestId = root.find(self.preffix + "RequestID").text

        if root.find(self.preffix + "BalanceRequest"):
            action = "BalanceRequest"
        checkRequest = root.find(self.preffix + "ChequeRequest")
        if checkRequest:
            opType = root.find(self.preffix + "OperationType").text
            if opType == "Sale":
                action = "ChequeRequest_" + checkRequest.get("ChequeType")
            else:
                action = "ChequeRequest_" + opType
 
        self.log.info("Действие: %s", action)
        return self.__getResult("%s" % action, requestId)

    def __getResult(self, methodPath: str, requestId):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        responseFile = str(self.cases[methodPath].get('response')) + '.xml'
        self.log.info("Ответ : %s", responseFile)

        namespaces = dict([node for _, node in ET.iterparse(os.path.join(self.path, responseFile), events=['start-ns'])])
        for ns in namespaces:
            ET.register_namespace(ns, namespaces[ns])
        tree = ET.parse(os.path.join(self.path, responseFile))
        root = tree.getroot()
        reqId = root.find(self.preffix + 'RequestID')
        reqId.text = requestId
        tree.write(os.path.join(self.path, responseFile), encoding = 'utf-8')

        fo = open(os.path.join(self.path, responseFile), "r+")
        response = fo.read()

        time.sleep(self.cases[methodPath].get('timeout', 0))

        response = bottle.HTTPResponse(status=200, body=response)
        response.add_header("Content-Type", "application/xml")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(self.path, 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Manzana2019()
