from configparser import ConfigParser
from typing import Dict, Iterable, Any
import json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Markdown(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]
    servicePath: str

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        self.servicePath = conf.get('markdown', 'path') if conf.has_option('markdown', 'path') else '/exciseMark/status'

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса %s' % type(self).__name__)
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error("Ошибка загрузки тест-кейсов: %s" % str(e))

        servicestub.route(self.servicePath, method='POST', callback=self.markdown)

    def markdown(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))

        if self.servicePath not in self.cases:
            raise bottle.HTTPError(400, 'В тест-кейсах нет значений')

        if not len(self.cases[self.servicePath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[self.servicePath][0]:
            return json.dumps(self.cases[self.servicePath][0].get('response'))

        self.cases[self.servicePath][0]['count'] -= 1
        response = json.dumps(self.cases[self.servicePath][0].get('response'))

        if self.cases[self.servicePath][0]['count'] == 0:
            del self.cases[self.servicePath][0]

        return response


instance = Markdown()
