from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class SberSbp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса sbersbp (platiqr3)')
        self.__updateCases()

        servicestub.route('/tokens/v2/oauth', method='POST', callback=self.token)
        servicestub.route('/qr/order/v3/status', method='POST', callback=self.status)
        servicestub.route('/qr/order/v3/creation', method='POST', callback=self.creation)
        servicestub.route('/qr/order/v3/cancel', method='POST', callback=self.cancel)

    def token(self):
        self.log.info("Запрос токена")
        self.__updateCases()
        return self.__getResult('/tokens/v2/oauth')

    def status(self):
        self.log.info("Запрос статуса")
        self.__updateCases()
        return self.__getResult("/qr/order/v3/status")

    def creation(self):
        self.log.info("Создание заказа")
        self.__updateCases()
        return self.__getResult("/qr/order/v3/creation")

    def cancel(self):
        self.log.info("Отмена оплаты")
        self.__updateCases()
        return self.__getResult("/qr/order/v3/cancel")

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))

        responseSuffix = self.cases[methodPath].get('responseSuffix', '')

        body = json.dumps(self.cases[methodPath].get('response' + responseSuffix))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in "sbersbp", "status":
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = SberSbp()
