from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Verification(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route('/CSrest/rest/ident/cards', method='GET', callback=self.verification)

    def verification(self):
        self.__updateCases()

        params = dict(bottle.request.query.decode())
        cardNumber = params.get('number', '')
        cardGroupId = params.get('idcardgroup', 0)

        self.log.info("Верификация карты: '%s'" % cardNumber)
        methodPath = '/CSrest/rest/ident/cards'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'verification.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Verification()
